@@ -76,15 +76,15 @@ class HttpRegister(typing.Generic[ViewType]):
         """
 
         def register(endpoint: ViewType) -> ViewType:
-            route: HttpRoute[ViewType] = reduce(
-                operator.matmul,
-                middlewares,
-                self._http_route(path, endpoint, name, summary, description, tags),
+            route: HttpRoute[ViewType] = self._http_route(
+                path, endpoint, name, summary, description, tags
             )
             if method != "any":
                 route = route @ self._required_method(method.upper())
 
-            _ = self.__routes << route
+            reduce(operator.matmul, middlewares, route)
+
+            self.__routes <<= route
             return endpoint
 
         return register
@@ -534,7 +534,7 @@ class Router(RouteRegisterMixin[ViewType], typing.Generic[ViewType]):
     def url_for(
         self,
         name: str,
-        path_params: typing.Dict[str, typing.Any] = {},
+        path_params: typing.Mapping[str, typing.Any] = {},
     ) -> str:
         if name not in self.routes_mapping:
             raise NoRouteFound(f"No route with name '{name}' exists")
