@@ -1,7 +1,8 @@
 import json
 from http import HTTPStatus
-from typing import Any, List
+from typing import Any, List, Tuple
 
+import httpx
 import pytest
 from async_asgi_testclient import TestClient
 from pydantic import BaseModel
@@ -18,6 +19,9 @@ from kui.asgi import (
     OpenAPI,
     Path,
     Routes,
+    api_key_auth_dependency,
+    basic_auth,
+    bearer_auth,
     required_method,
 )
 
@@ -106,9 +110,11 @@ async def test_openapi_page():
             "/hello": {
                 "get": {
                     "summary": "hello",
+                    "description": "hello",
                     "responses": {
                         "200": {
                             "description": "Request fulfilled, document follows",
+                            "headers": {},
                             "content": {
                                 "application/json": {
                                     "schema": {
@@ -129,6 +135,7 @@ async def test_openapi_page():
                         "200": {
                             "description": "Request fulfilled, document follows",
                             "content": {"text/html": {"schema": {"type": "string"}}},
+                            "headers": {},
                         }
                     },
                 },
@@ -138,6 +145,7 @@ async def test_openapi_page():
                     "responses": {
                         "201": {
                             "description": "Document created, URL follows",
+                            "headers": {},
                             "content": {
                                 "application/json": {
                                     "schema": {
@@ -175,6 +183,7 @@ async def test_openapi_page():
                     "responses": {
                         "422": {
                             "description": "",
+                            "headers": {},
                             "content": {
                                 "application/json": {
                                     "schema": {
@@ -226,6 +235,8 @@ async def test_openapi_page():
             },
             "/middleware/http-view": {
                 "get": {
+                    "summary": "...",
+                    "description": "......",
                     "parameters": [
                         {
                             "in": "header",
@@ -237,77 +248,17 @@ async def test_openapi_page():
                         }
                     ],
                     "responses": {
-                        "422": {
-                            "description": "",
-                            "content": {
-                                "application/json": {
-                                    "schema": {
-                                        "type": "array",
-                                        "items": {
-                                            "type": "object",
-                                            "properties": {
-                                                "loc": {
-                                                    "title": "Location",
-                                                    "description": "error field",
-                                                    "type": "array",
-                                                    "items": {"type": "string"},
-                                                },
-                                                "type": {
-                                                    "title": "Type",
-                                                    "description": "error type",
-                                                    "type": "string",
-                                                },
-                                                "msg": {
-                                                    "title": "Message",
-                                                    "description": "error message",
-                                                    "type": "string",
-                                                },
-                                                "ctx": {
-                                                    "title": "Context",
-                                                    "description": "error context",
-                                                    "type": "string",
-                                                },
-                                                "in": {
-                                                    "title": "In",
-                                                    "type": "string",
-                                                    "enum": [
-                                                        "path",
-                                                        "query",
-                                                        "header",
-                                                        "cookie",
-                                                        "body",
-                                                    ],
-                                                },
-                                            },
-                                            "required": ["loc", "type", "msg"],
-                                        },
-                                    }
-                                }
-                            },
-                        },
                         "200": {
                             "description": "Request fulfilled, document follows",
                             "content": {"text/html": {"schema": {"type": "string"}}},
+                            "headers": {},
                         },
                         "401": {
                             "description": "No permission -- see authorization schemes"
                         },
-                    },
-                },
-                "post": {
-                    "parameters": [
-                        {
-                            "in": "header",
-                            "name": "authorization",
-                            "description": "JWT Token",
-                            "required": True,
-                            "schema": {"title": "Authorization", "type": "string"},
-                            "deprecated": False,
-                        }
-                    ],
-                    "responses": {
                         "422": {
                             "description": "",
+                            "headers": {},
                             "content": {
                                 "application/json": {
                                     "schema": {
@@ -354,8 +305,25 @@ async def test_openapi_page():
                                 }
                             },
                         },
+                    },
+                },
+                "post": {
+                    "summary": "...",
+                    "description": "......",
+                    "parameters": [
+                        {
+                            "in": "header",
+                            "name": "authorization",
+                            "description": "JWT Token",
+                            "required": True,
+                            "schema": {"title": "Authorization", "type": "string"},
+                            "deprecated": False,
+                        }
+                    ],
+                    "responses": {
                         "201": {
                             "description": "Document created, URL follows",
+                            "headers": {},
                             "content": {
                                 "application/json": {
                                     "schema": {
@@ -371,22 +339,9 @@ async def test_openapi_page():
                         "401": {
                             "description": "No permission -- see authorization schemes"
                         },
-                    },
-                },
-                "delete": {
-                    "parameters": [
-                        {
-                            "in": "header",
-                            "name": "authorization",
-                            "description": "JWT Token",
-                            "required": True,
-                            "schema": {"title": "Authorization", "type": "string"},
-                            "deprecated": False,
-                        }
-                    ],
-                    "responses": {
                         "422": {
                             "description": "",
+                            "headers": {},
                             "content": {
                                 "application/json": {
                                     "schema": {
@@ -433,10 +388,75 @@ async def test_openapi_page():
                                 }
                             },
                         },
+                    },
+                },
+                "delete": {
+                    "summary": "...",
+                    "description": "......",
+                    "parameters": [
+                        {
+                            "in": "header",
+                            "name": "authorization",
+                            "description": "JWT Token",
+                            "required": True,
+                            "schema": {"title": "Authorization", "type": "string"},
+                            "deprecated": False,
+                        }
+                    ],
+                    "responses": {
                         "204": {"description": "Request fulfilled, nothing follows"},
                         "401": {
                             "description": "No permission -- see authorization schemes"
                         },
+                        "422": {
+                            "description": "",
+                            "headers": {},
+                            "content": {
+                                "application/json": {
+                                    "schema": {
+                                        "type": "array",
+                                        "items": {
+                                            "type": "object",
+                                            "properties": {
+                                                "loc": {
+                                                    "title": "Location",
+                                                    "description": "error field",
+                                                    "type": "array",
+                                                    "items": {"type": "string"},
+                                                },
+                                                "type": {
+                                                    "title": "Type",
+                                                    "description": "error type",
+                                                    "type": "string",
+                                                },
+                                                "msg": {
+                                                    "title": "Message",
+                                                    "description": "error message",
+                                                    "type": "string",
+                                                },
+                                                "ctx": {
+                                                    "title": "Context",
+                                                    "description": "error context",
+                                                    "type": "string",
+                                                },
+                                                "in": {
+                                                    "title": "In",
+                                                    "type": "string",
+                                                    "enum": [
+                                                        "path",
+                                                        "query",
+                                                        "header",
+                                                        "cookie",
+                                                        "body",
+                                                    ],
+                                                },
+                                            },
+                                            "required": ["loc", "type", "msg"],
+                                        },
+                                    }
+                                }
+                            },
+                        },
                     },
                 },
             },
@@ -493,16 +513,16 @@ def test_openapi_single_function_summary_and_description():
         """
         return ""
 
-    assert openapi._generate_path(app, app.router.search("http", "/0")[1], "/")[0] == {
+    assert openapi._generate_path(app, app.router.search("http", "/0")[1], "/") == {
         "get": {"summary": "Summary", "description": "Description"}
     }
-    assert openapi._generate_path(app, app.router.search("http", "/1")[1], "/")[0] == {
+    assert openapi._generate_path(app, app.router.search("http", "/1")[1], "/") == {
         "get": {"summary": "Summary"}
     }
-    assert openapi._generate_path(app, app.router.search("http", "/2")[1], "/")[0] == {
+    assert openapi._generate_path(app, app.router.search("http", "/2")[1], "/") == {
         "get": {"summary": "Summary", "description": "Description"}
     }
-    assert openapi._generate_path(app, app.router.search("http", "/3")[1], "/")[0] == {
+    assert openapi._generate_path(app, app.router.search("http", "/3")[1], "/") == {
         "get": {"summary": "Summary", "description": "Description"}
     }
 
@@ -517,8 +537,7 @@ def test_openapi_single_function_tags():
         return ""
 
     assert openapi._generate_path(app, app.router.search("http", "/")[1], "/") == (
-        {"get": {"tags": ["tag0"]}},
-        {},
+        {"get": {"tags": ["tag0"]}}
     )
 
 
@@ -535,11 +554,111 @@ def test_openapi_routes_tags():
     )
 
     assert openapi._generate_path(app, app.router.search("http", "/")[1], "/") == (
-        {"get": {"tags": ["tag0"]}},
-        {},
+        {"get": {"tags": ["tag0"]}}
     )
 
 
+@pytest.mark.asyncio
+async def test_openapi_security():
+    app = Kui()
+    openapi = OpenAPI()
+    app.router <<= "/docs" // openapi.routes
+
+    @app.router.http.get("/basic")
+    async def basic(user_and_password: Annotated[Tuple[str, str], Depends(basic_auth)]):
+        return ", ".join(user_and_password)
+
+    @app.router.http.get("/bearer")
+    async def bearer(token: Annotated[str, Depends(bearer_auth)]):
+        return token
+
+    @app.router.http.get("/api-key")
+    async def api_key(
+        api_key: Annotated[str, Depends(api_key_auth_dependency("api-key"))]
+    ):
+        return api_key
+
+    async with httpx.AsyncClient(app=app, base_url="http://testserver") as client:
+        resp = await client.get("/docs/json")
+        openapi_json = resp.json()
+
+        assert openapi_json == {
+            "openapi": "3.0.3",
+            "info": {"title": "Kuí API", "version": "1.0.0"},
+            "paths": {
+                "/basic": {
+                    "get": {
+                        "security": [{"BasicAuth": []}],
+                        "responses": {
+                            "401": {
+                                "description": "No permission -- see authorization schemes",
+                                "headers": {
+                                    "WWW-Authenticate": {
+                                        "description": "Basic authentication",
+                                        "schema": {"type": "string"},
+                                    }
+                                },
+                            }
+                        },
+                    }
+                },
+                "/bearer": {
+                    "get": {
+                        "security": [{"BearerAuth": []}],
+                        "responses": {
+                            "401": {
+                                "description": "No permission -- see authorization schemes",
+                                "headers": {
+                                    "WWW-Authenticate": {
+                                        "description": "Bearer token",
+                                        "schema": {"type": "string"},
+                                    }
+                                },
+                            }
+                        },
+                    }
+                },
+                "/api-key": {
+                    "get": {
+                        "security": [{"ApiKeyAuth": []}],
+                        "responses": {
+                            "401": {
+                                "description": "No permission -- see authorization schemes"
+                            }
+                        },
+                    }
+                },
+            },
+            "tags": [],
+            "components": {
+                "securitySchemes": {
+                    "BasicAuth": {"type": "http", "scheme": "basic"},
+                    "BearerAuth": {"type": "http", "scheme": "bearer"},
+                    "ApiKeyAuth": {"type": "apiKey", "name": "api-key", "in": "header"},
+                },
+                "schemas": {},
+            },
+            "servers": [
+                {"url": "/", "description": "Current server"},
+                {
+                    "url": "{scheme}://{address}/",
+                    "description": "Custom API Server Host",
+                    "variables": {
+                        "scheme": {
+                            "default": "http",
+                            "enum": ["http", "https"],
+                            "description": "http or https",
+                        },
+                        "address": {
+                            "default": "testserver",
+                            "description": "api server's host[:port]",
+                        },
+                    },
+                },
+            ],
+        }
+
+
 def test_openapi_depend_response():
     app = Kui()
     openapi = OpenAPI()
@@ -570,6 +689,7 @@ def test_openapi_depend_response():
                 "responses": {
                     "422": {
                         "description": "",
+                        "headers": {},
                         "content": {
                             "application/json": {
                                 "schema": {
@@ -617,10 +737,47 @@ def test_openapi_depend_response():
                         },
                     },
                     "401": {
-                        "description": "No permission -- see authorization schemes"
+                        "description": "No permission -- see authorization schemes",
                     },
                 },
             }
-        },
-        {},
+        }
     )
+
+
+def test_openapi_multi_type_responses():
+    app = Kui()
+    openapi = OpenAPI()
+    app.router <<= "/docs" // openapi.routes
+
+    class Message(BaseModel):
+        message: str
+
+    @app.router.http.get("/")
+    async def homepage() -> Annotated[
+        Any, JSONResponse[200, {}, Message], HTMLResponse[200]
+    ]:
+        pass
+
+    assert openapi._generate_path(app, app.router.search("http", "/")[1], "/") == {
+        "get": {
+            "responses": {
+                "200": {
+                    "description": "Request fulfilled, document follows",
+                    "headers": {},
+                    "content": {
+                        "application/json": {
+                            "schema": {
+                                "type": "object",
+                                "properties": {
+                                    "message": {"title": "Message", "type": "string"}
+                                },
+                                "required": ["message"],
+                            }
+                        },
+                        "text/html": {"schema": {"type": "string"}},
+                    },
+                }
+            }
+        }
+    }
